/*---------------------------------------------------------*\
| InstantMouseController.h                                  |
|                                                           |
|   Driver for Instant mouse                                |
|                                                           |
|   shafiahaz2478                               29 Aug 2024 |
|                                                           |
|   This file is part of the OpenRGB project                |
|   SPDX-License-Identifier: GPL-2.0-only                   |
\*---------------------------------------------------------*/
#pragma once

#define INSTANTMOUSEDEVICES_H

#define INSTANT_MICROELECTRONICS_VID            0x30FA
#define ADVANCED_GTA_250_PID                    0x1030
#define ANKO_KM43243952_VID                     0x1440
#define ANKO_KM43277483_VID                     0x1540
#define ANTESPORTS_GM600_PID                    0x1040
